/** Teardrop - metaRank.h

@brief This file is designed to launch multiple engines query via threads.
@brief It also gives a way to sort the results.

@author Olivier COUPELON (olivier.coupelon@wanadoo.fr)
@date 07/08/2006

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef METARANK_H
#define METARANK_H

#include <iostream>
#include <stdlib.h>
#include <sstream>
#include <algorithm>
#include <cctype>
#include "threadPool.h"
#include "pageParser.h"
#include "row.h"
#include "engineResults.h"
#include "tdParam.h"

using namespace std;

struct compareRow {
    compareRow(string & s) {
	if (s == "number") num = true;
	else num = false;
	field = s;
    }
    bool operator()(/*const */row r1, /*const */row r2) const {
        if (num) return r1.getNum() < r2.getNum();
        string s1 = r1.getFields()[field];
        transform(s1.begin(),s1.end(),s1.begin(),(int (*)(int)) toupper);
        string s2 = r2.getFields()[field];
        transform(s2.begin(),s2.end(),s2.begin(),(int (*)(int)) toupper);
        return s1 < s2;
        //return r1.getFields()[field] < r2.getFields()[field];
    }
    bool num;
    string field;
};

class metaRank {
public:
    metaRank(string query, list<string> &, string limit, tdParam &, string m = "simple");
    ~metaRank();

    /** This method executes the search on the given engines.
    For each engine, a thread is created, which enables query to be
    processed in parallel.
    */   
    void startParsing();
    
    /** This method launches the specified ranking algorithm.
    */   
    void startRanking();
    
    /** Check if some pageParser are still working
    */
    bool isAlive();

    /** Wait for all the pageParser to finish
    */
    void joinAll();

    /** The list of results finaly obtained
    */
    list<row> & getResults();
    
    
    /** Gives the number of thread already finished. Usefull for progress bar.
    */
    int howManyFinished();
    
    /** Stops the current threads by cancelling them.
    */
    void stop();
        
    /** The number of results obtained so far.
    @param if given, it is returned unless more results than old have been found.
    Usefull to get a strictly non decreasing number as result number can decrease
    when ranking is started, due to multiple same results.
    */
    int getResultsNumber(int old = 0);
    
    /** This is a simple ranking method.
    I takes the first row of the first engine, and if not already included
    in the ranked list pushes it back. If it has already been included,
    the existing raw is added the new engine. The algorithm then switches
    to the next engine until every engineResults are empty.
    */   
    void simpleRank();
    
    /** A simple method used to display the final results.
    */
    void toString();
    
    /** A more advanced method to display only required fields   
    */
    void toString(string, string);
    
    /** Sort the given list of row in order of the given field
    */
    static void sort(list<row> & lr,string field);
    
private:
    threadPool<pageParser> threads;
    list<pageParser *> pplist;
    list<engineResults> *global_results;
    list<row> ranked_results;
    string query;
    list<string> engines;
    unsigned long limit;
    string method;
    static string intToString(int);
    tdParam tdp;
};

#endif
